"
InstVarMappingArray is used in instance migration to control
 migration of both named and dynamic instanceVaribles.

 instance variables  
   preserveDynamic    aBoolean

   varying instanceVariables 
     1: a SmallInteger,  offset of instVar in old object corresponding to instVarAt: 1 in new object
     ...

"
Class {
	#name : 'InstVarMappingArray',
	#superclass : 'Array',
	#instVars : [
		'dynamicIvsToNamed',
		'namedIvsToDynamic',
		'preserveDynamic'
	],
	#category : nil
}

{ #category : 'Instance Creation' }
InstVarMappingArray class >> mappingFrom: oldClass to: newClass [
 "Creates an instance that preserves dynamic instVars of instances of oldClass.
  Named instVars in oldClass that do not exist in newClass are not preserved.

  Use InstVarMappingArray class >> new  followed by
      InstVarMappingArray >> initializeFrom:to:preserveDynamic:
  to customize a mapping . "

^ self new initializeFrom: oldClass to: newClass

]

{ #category : 'Initialization' }
InstVarMappingArray >> initializeFrom: oldClass to: newClass [

^ self initializeFrom: oldClass to: newClass preserveDynamic: true

]

{ #category : 'Initialization' }
InstVarMappingArray >> initializeFrom: oldClass to: newClass preserveDynamic: aBoolean [
  "Initializes the receiver for subsequent use as receiver of copyIvsFrom:to: .

   If aBoolean==true, dynamic instance variables are preserved in the
   new instance, if false, dynamic instance variables are not preserved. "

  preserveDynamic := aBoolean .
  self addAll: (newClass instVarMappingTo: oldClass) .

]

{ #category : 'Accessing' }
InstVarMappingArray >> preserveDynamic [
  ^ preserveDynamic

]

{ #category : 'Updating' }
InstVarMappingArray >> preserveDynamic: aBoolean [
  preserveDynamic := aBoolean

]
